import logging

from parallels.operability_check import DomainIssue
from parallels.operability_check import Severity, DNSIssueType
from parallels.operability_check.utils.dns_utils import DnsRequestTimeoutException, get_difference

logger = logging.getLogger(__name__)

class DNSChecker:
	def check(
		self, domains_to_check, 
		max_difference_lines=3, save_external_report_data_function=None, 
		dns_server_name='PPA DNS server',
		difference_file_panel_id='ppa'
	):
		issues = []

		def add_issue(severity, category, problem):
			issues.append(DomainIssue(domain_name=domain_to_check.domain_name, severity=severity, category=category, problem=problem))

		for domain_to_check in domains_to_check:
			for dns_server_ip in domain_to_check.dns_servers:
				try:
					difference = get_difference(domain_to_check.dns_records, dns_server_ip)

					if len(difference) > 0:
						if len(difference) <= max_difference_lines or save_external_report_data_function is None: # put all differences to screen immediately
							difference_str = "Here is the difference:\n%s" % (u"\n".join(difference))
						else: # put first 3 differences to screen as an example, put all other differences into a separate file
							filename = save_external_report_data_function(
								"dns.%s.%s.difference.%s" % (
									difference_file_panel_id, dns_server_ip, domain_to_check.domain_name
								), 
								u"The DNS server at %s returned incorrect DNS records:\n%s\n" % (
									dns_server_ip,
									u"\n".join(difference)
								)
							)
							difference_str = "Here is an example of differing records:\n%s\nAll information about the differences is saved to the file %s" % (u"\n".join(difference[:3]), filename)

						errmsg = "The DNS server at %s returned incorrect DNS records. %s" % (dns_server_ip, difference_str,)
						add_issue(
							severity=Severity.ERROR, category=DNSIssueType.DIFFERENT_RECORDS,
							problem=errmsg
						)
				except DnsRequestTimeoutException, e:
					logger.debug(u"Exception:", exc_info=e)
					errmsg = "The DNS query to the %s at %s was timed out." % (dns_server_name, e.server_ip)
					# do not return any report comparing records - just a timeout message
					add_issue(
						severity=Severity.ERROR, category=DNSIssueType.DNS_SERVER_TIMEOUT,
						problem=errmsg
					)
				except KeyboardInterrupt:
					# for compatibility with python 2.4
					raise
				except Exception, e:
					logger.debug(u"Exception:", exc_info=e)
					# do not return any report comparing records - just a general failure message
					add_issue(
						severity=Severity.WARNING, category=DNSIssueType.INTERNAL_ERROR,
						problem=u"An internal error occurred: %s\nUnable to check whether the DNS zone is resolved by the %s at %s." % (str(e), dns_server_name, dns_server_ip)
					)
		return issues

