import errno
import urllib2
import urlparse
import httplib
import logging

logger = logging.getLogger(__name__)

#
# We use NSS due to missing SSL SNI support in OpenSSL library on CentOS/RHEL 5.
# It's required to successfully check windows 2012 IIS domains from CentOS/RHEL 5 PPA MN.
#
nss_imported = False;
try:
	import nss.io as io
	import nss.nss as nss
	import nss.ssl as ssl
	nss_imported = True;
except ImportError:
	logger.debug(u'Unable to import NSS. We will use OpenSSL')

nss_is_initialized = False

class HttpClient(object):
	def get(self, url, host):
		parsed_url = urlparse.urlparse(url)
		if self._is_ipv6(host):
			host = '[%s]' % host

		replaced_url = (parsed_url[0], host, parsed_url[2], parsed_url[3], parsed_url[4], parsed_url[5])
		url = urlparse.urlunparse(replaced_url)

		request = urllib2.Request(url)
		hostname = parsed_url[1].split(':')[0]  # omit port if netloc contains it, for example: "example.com:80"
		request.add_header('Host', hostname)

		if nss_imported:
			opener = urllib2.build_opener(NoRedirection, HTTPSHandler)
			response = opener.open(request)
		else:
			opener = urllib2.build_opener(NoRedirection, urllib2.HTTPSHandler)
			try:
				response = opener.open(request)
			except urllib2.URLError, e:
				# Workaround for https://jira.sw.ru/browse/PMT-147 (
				# Unable to check HTTPS on site hosting on Windows 2012 if migration checker run on Windows)
				if parsed_url[0].lower() == 'https':
					return HttpResponse(200)
				else:
					raise


		return HttpResponse(code=response.code)

	@staticmethod
	def _is_ipv6(string):
		# We cannot use socket.inet_pton because it is not available on Windows and may be some Unix OSes
		# https://docs.python.org/2/library/socket.html#socket.inet_pton
		# Availability: Unix (maybe not all platforms).
		return ':' in string

class HTTPSHandler(urllib2.HTTPSHandler):
	def https_open(self, req):
		return self.do_open(NSSConnection, req)

class NoRedirection(urllib2.HTTPErrorProcessor):
	def http_response(self, request, response):
		return response

	https_response = http_response

class HttpResponse(object):
	def __init__(self, code):
		self.code = code

class NSSConnection(httplib.HTTPSConnection):
	NSS_CERT_DB = '/etc/pki/nssdb'
	SOCKET_TIMEOUT = 30

	def __init__(self, host, port=None, strict=None, db_dir=NSS_CERT_DB, **kwargs):
		httplib.HTTPSConnection.__init__(self, host, port, strict, **kwargs)

		if not db_dir:
			raise RuntimeError('db_dir is required')

		if not nss_is_initialized:
			nss.nss_init(db_dir)

		self.sock = None
		self._hostname = self.host
		ssl.set_domestic_policy()

	def connect(self):
		addr_info = io.AddrInfo(self.host)
		for net_addr in addr_info:
			net_addr.port = self.port
			self._create_socket(net_addr.family)
			try:
				self.sock.connect(net_addr, timeout=io.seconds_to_interval(self.SOCKET_TIMEOUT))
				return
			except Exception, e:
				logger.debug(u'Exception:', exc_info=e)

		raise IOError(errno.ENOTCONN, u'Could not connect to %s at port %d' % (self.host, self.port))

	def _create_socket(self, family):
		# We accept any server's SSL certificate
		def auth_certificate_callback(sock, check_sig, is_server, certdb):
			return True

		# We do not use client SSL certificates
		def client_auth_data_callback(ca_names, chosen_nickname, password, certdb):
			return False

		self.sock = ssl.SSLSocket(family)
		self.sock.set_ssl_option(ssl.SSL_SECURITY, True)
		self.sock.set_ssl_option(ssl.SSL_HANDSHAKE_AS_CLIENT, True)
		self.sock.set_hostname(self._hostname)

		self.sock.set_auth_certificate_callback(auth_certificate_callback, nss.get_default_certdb())
		self.sock.set_client_auth_data_callback(client_auth_data_callback, '', '', nss.get_default_certdb())

	def _send_request(self, method, url, body, headers):
		host_header_value = [v for k, v in headers.iteritems() if k.lower() == 'host']
		if host_header_value:
			self._hostname = host_header_value[0]

		httplib.HTTPSConnection._send_request(self, method, url, body, headers)
